@@ -89,6 +89,7 @@
 ;;   - Maybe get rid of second " in reader macro. #"toString looks nicer, but might 
 ;;     confuse lisp mode.
 ;;   - write jmap, analogous to map, but can take java collections, java arrays etc.
+;;      In progress with jss-3.5.0's JSS:MAP
 ;;   - write loop clauses for java collections. 
 ;;   - Register classes in .class files below classpath directories (when :wild-inferiors works)
 ;;   - Make documentation like Edi Weitz
@@ -121,11 +122,11 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defvar *do-auto-imports* t 
-    "Whether to automatically introspect all Java classes on the classpath when JSS is loaded."))
-
-(defvar *muffle-warnings* t)
-
-(defvar *imports-resolved-classes* (make-hash-table :test 'equalp))
+    "Whether to automatically introspect all Java classes on the classpath when JSS is loaded.")
+  (defvar *muffle-warnings* t
+    "Attempt to make JSS less chatting about how things are going.")
+  (defvar *imports-resolved-classes* (make-hash-table :test 'equalp)
+    "Hashtable of all resolved imports by the current process."))
 
 (defun find-java-class (name)
   "Returns the java.lang.Class representation of NAME.
@@ -205,14 +206,18 @@ NAME can either string or a symbol according to the usual JSS conventions."
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defun read-invoke (stream char arg) 
-    (unread-char char stream)
-    (let ((name (read stream)))
-      (if (or (find #\. name) (find #\{ name))
-          (jss-transform-to-field name)
-          (let ((object-var (gensym))
-                (args-var (gensym)))
-            `(lambda (,object-var &rest ,args-var) 
-               (invoke-restargs ,name  ,object-var ,args-var ,(eql arg 0)))))))
+    (if (eql arg 1)
+        (progn (asdf:make 'javaparser)
+               (read-sharp-java-expression stream))
+        (progn
+          (unread-char char stream)
+          (let ((name (read stream)))
+            (if (or (find #\. name) (find #\{ name))
+                (jss-transform-to-field name arg)
+                (let ((object-var (gensym))
+                      (args-var (gensym)))
+                  `(lambda (,object-var &rest ,args-var) 
+                     (invoke-restargs ,name  ,object-var ,args-var ,(eql arg 0)))))))))
   (set-dispatch-macro-character #\# #\" 'read-invoke))
 
 (defmacro with-constant-signature (fname-jname-pairs &body body)
@@ -245,10 +250,28 @@ want to avoid the overhead of the dynamic dispatch."
                (with-constant-signature ,(cdr fname-jname-pairs)
                  ,@body)))))))
 
+(defvar *class-lookup-overrides*)
+
+(defmacro with-class-lookup-disambiguated (overrides &body body)
+  "Suppose you have code that references class using the symbol 'object, and this is ambiguous. E.g. in my system java.lang.Object, org.omg.CORBA.Object. Use (with-class-lookup-disambiguated (lang.object) ...). Within dynamic scope, find-java-class first sees if any of these match, and if so uses them to lookup the class."
+  `(let ((*class-lookup-overrides* ',overrides))
+     ,@body))
+
+(defun maybe-found-in-overridden (name)
+  (when (boundp '*class-lookup-overrides*)
+      (let ((found (find-if (lambda(el) (#"matches" (string el) (concatenate 'string "(?i).*" (string name) "$")))
+                            *class-lookup-overrides*)))
+        (if found
+            (let ((*class-lookup-overrides* nil))
+              (lookup-class-name found))))))
+
+
 (defun lookup-class-name (name &key
                                  (table *class-name-to-full-case-insensitive*)
-                                 (muffle-warning nil)
+                                 (muffle-warning *muffle-warnings*)
                                  (return-ambiguous nil))
+  (let ((overridden (maybe-found-in-overridden name)))
+    (when overridden (return-from lookup-class-name overridden)))
   (setq name (string name))
   (let* (;; cant (last-name-pattern (#"compile" '|java.util.regex.Pattern| ".*?([^.]*)$"))
          ;; reason: bootstrap - the class name would have to be looked up...
@@ -268,11 +291,11 @@ want to avoid the overhead of the dynamic dispatch."
                          (length end))
                       (length full)))
                  (ambiguous (choices)
-		   (if return-ambiguous 
-		       (return-from lookup-class-name choices)
-		       (error "Ambiguous class name: ~a can be ~{~a~^, ~}" name choices))))
+                   (if return-ambiguous 
+                       (return-from lookup-class-name choices)
+                       (error "Ambiguous class name: ~a can be ~{~a~^, ~}" name choices))))
             (if (zerop bucket-length)
-		(progn (unless muffle-warning (warn "can't find class named ~a" name)) nil)
+                (progn (unless muffle-warning (warn "can't find class named ~a" name)) nil)
                 (let ((matches (loop for el in bucket when (matches-end name el 'char=) collect el)))
                   (if (= (length matches) 1)
                       (car matches)
@@ -281,10 +304,11 @@ want to avoid the overhead of the dynamic dispatch."
                             (if (= (length matches) 1)
                                 (car matches)
                                 (if (= (length matches) 0)
-				    (progn (unless muffle-warning (warn "can't find class named ~a" name)) nil)
+                                    (progn (unless muffle-warning (warn "can't find class named ~a" name)) nil)
                                     (ambiguous matches))))
                           (ambiguous matches))))))))))
 
+#+(or)
 (defun get-all-jar-classnames (jar-file-name)
   (let* ((jar (jnew (jconstructor "java.util.jar.JarFile" (jclass "java.lang.String")) (namestring (truename jar-file-name))))
          (entries (#"entries" jar)))
@@ -303,7 +327,35 @@ want to avoid the overhead of the dynamic dispatch."
                   (matcher (matcher name-pattern fullname))
                   (name (progn (matches matcher) (group matcher 1))))
              (cons name fullname))
-           ))))
+            ))))
+#| 
+ Under openjdk11 this is around 10x slower than
+
+(list (time (jss::get-all-jar-classnames "/Users/evenson/work/abcl-jdk11/dist/abcl.jar"))
+      (time (jss::%get-all-jar-classnames "/Users/evenson/work/abcl-jdk11/dist/abcl.jar")))
+
+0.034 seconds real time
+2268 cons cells
+0.12 seconds real time
+209164 cons cells
+|#
+(defun get-all-jar-classnames (jar-pathname-or-string)
+  (let* ((jar
+           (if (ext:pathname-jar-p jar-pathname-or-string)
+               jar-pathname-or-string
+               ;; better be a string
+               (ext:as-jar-pathname-archive jar-pathname-or-string)))
+         (entries
+           (directory (merge-pathnames "**/*" jar))))
+    (loop :for entry :in entries
+          :for name = (pathname-name entry)
+          :for type = (pathname-type entry)
+          :when (equal type "class")
+            :collect 
+            (cons
+             name
+             ;;; Fully qualified classname be like 'org.armedbear.lisp.ArgumentListProcessor$ArgumentMatcher'
+             (format nil "~{~a.~}~a" (rest (pathname-directory entry)) name)))))
 
 (defun jar-import (file)
   "Import all the Java classes contained in the pathname FILE into the JSS dynamic lookup cache."
@@ -401,7 +453,6 @@ associated is used to look up the static FIELD."
 (defun (setf get-java-field) (value object field &optional (try-harder *running-in-osgi*))
   (set-java-field object field value try-harder))
 
-
 (defconstant +for-name+ 
   (jmethod "java.lang.Class" "forName" "java.lang.String" "boolean" "java.lang.ClassLoader"))
 
@@ -419,6 +470,45 @@ associated is used to look up the static FIELD."
     (format stream "method ~a" (#"toString" obj))))
 
 (defun do-auto-imports ()
+  (if (sys::system-artifacts-are-jars-p)
+      (do-auto-imports-from-jars)
+      (progn
+        ;;; First, import all the classes available from the module system
+        (do-auto-imports-from-modules)
+        ;;; Then, introspect any jars that appear on the classpath
+        (loop :for entry :in (second (multiple-value-list (sys::java.class.path)))
+             :doing (let ((p (pathname entry)))
+                      (when (string-equal (pathname-type p) "jar")
+                        (jar-import p)))))))
+
+(defun do-auto-imports-from-modules ()
+  (loop :for (name . full-class-name) :in (all-class-names-from-modules)
+     :doing
+       (pushnew full-class-name (gethash name *class-name-to-full-case-insensitive*) 
+                :test 'equal)))
+
+(defun all-class-names-from-modules ()
+  (let ((class-pattern (jstatic "compile" "java.util.regex.Pattern" ".*\\.class$"))
+        (name-pattern (jstatic "compile" "java.util.regex.Pattern" ".*?([^.]*)$")))
+    (loop
+       :for module :across (chain (jstatic "boot" "java.lang.ModuleLayer")
+                                  "configuration" "modules" "stream" "toArray")
+       :appending
+         (loop
+            :for class-as-path :across (chain module "reference" "open" "list" "toArray")
+            :when
+              (jcall "matches" (jcall "matcher" class-pattern class-as-path))
+            :collect
+              (let* ((full-name (jcall "substring" (jcall "replace" class-as-path #\/ #\.)
+                                           0
+                                           (- (jcall "length" class-as-path) (jcall "length" ".class"))))
+                     (matcher (jcall "matcher" name-pattern full-name))
+                     (name (progn
+                             (jcall "matches" matcher)
+                             (jcall "group" matcher 1))))
+                (cons name full-name))))))
+
+(defun do-auto-imports-from-jars ()
   (labels ((expand-paths (cp)
              (loop :for s :in cp
                 :appending (loop :for entry 
@@ -445,8 +535,11 @@ associated is used to look up the static FIELD."
               'cons))
            (do-imports (cp)
              (import-classpath (expand-paths (split-classpath cp)))))
-    (do-imports (jcall "getClassPath" (jstatic "getRuntimeMXBean" '|java.lang.management.ManagementFactory|)))
-    (do-imports (jcall "getBootClassPath" (jstatic "getRuntimeMXBean" '|java.lang.management.ManagementFactory|)))))
+
+    (let ((mx-bean (jstatic "getRuntimeMXBean"
+                            '|java.lang.management.ManagementFactory|)))
+      (do-imports (jcall "getClassPath" mx-bean))
+      (do-imports (jcall "getBootClassPath" mx-bean)))))
 
 (eval-when (:load-toplevel :execute)
   (when *do-auto-imports* 
