@@ -16,10 +16,16 @@
    #:ensure-mvn-version
    
    #:find-mvn
+   #:mvn-version
 
    #:*mvn-directory*
 
    #:init
+   #:make-local-maven
+
+   ;; condition 
+   #:no-aether-maven-libs
+   
 
 ;;; "Internal" API
    #:resolve-dependencies
@@ -28,6 +34,7 @@
 ;;;; Maven 
    #:*mvn-libs-directory*
    #:*maven-http-proxy*
+   #:*default-repository*
    #:make-remote-repository
    #:*maven-remote-repository*
    #:resolve-multiple-maven-dependencies
