@@ -34,6 +34,7 @@
 
 package org.armedbear.lisp;
 
+import java.io.Serializable;
 import java.util.List;
 import java.util.ArrayList;
 import static org.armedbear.lisp.Lisp.*;
@@ -43,7 +44,7 @@ import static org.armedbear.lisp.Lisp.*;
  * The lambda list may either be of type ORDINARY or MACRO lambda list.
  * All other lambda lists are parsed elsewhere in our code base.
  */
-public class ArgumentListProcessor {
+public class ArgumentListProcessor implements Serializable {
     
   public enum LambdaListType {
       ORDINARY,
@@ -188,7 +189,7 @@ public class ArgumentListProcessor {
         
         if (remaining.car() == Symbol.AND_WHOLE) {
             if (type == LambdaListType.ORDINARY) {
-                error(new ProgramError("&WHOLE not allowed in ordinary lambda lists."));
+              program_error("&WHOLE not allowed in ordinary lambda lists.");
             } else {
                 // skip the &WHOLE <var> part of the lambda list
                 remaining = remaining.cdr().cdr();
@@ -203,9 +204,9 @@ public class ArgumentListProcessor {
               {
                 if (obj == Symbol.AND_WHOLE) {
                     if (type == LambdaListType.ORDINARY)
-                      error(new ProgramError("&WHOLE not allowed in ordinary lambda lists."));
+                      program_error("&WHOLE not allowed in ordinary lambda lists.");
                     else
-                      error(new ProgramError("&WHOLE must appear first in macro lambda list."));
+                      program_error("&WHOLE must appear first in macro lambda list.");
                 }
                 if (state == STATE_AUX)
                   {
@@ -223,24 +224,21 @@ public class ArgumentListProcessor {
                   {
                     if (_andKey)
                       {
-                        error(new ProgramError(
-                          "&REST/&BODY must precede &KEY."));
+                        program_error("&REST/&BODY must precede &KEY.");
                       }
                     if (type == LambdaListType.ORDINARY && obj == Symbol.AND_BODY)
-                      error(new ProgramError("&BODY not allowed in ordinary lambda lists."));
+                      program_error("&BODY not allowed in ordinary lambda lists.");
                     state = STATE_REST;
                     arity = -1;
                     maxArgs = -1;
                     remaining = remaining.cdr();
                     if (remaining == NIL)
                       {
-                        error(new ProgramError(
-                          "&REST/&BODY must be followed by a variable."));
+                        program_error("&REST/&BODY must be followed by a variable.");
                       }
                     if (restVar != null) 
                       {
-                        error(new ProgramError(
-                          "&REST/&BODY may occur only once."));
+                        program_error("&REST/&BODY may occur only once.");
                       }
                     final LispObject remainingcar =  remaining.car();
                     if (remainingcar instanceof Symbol)
@@ -250,14 +248,13 @@ public class ArgumentListProcessor {
                       }
                     else
                       {
-                        error(new ProgramError(
-                          "&REST/&BODY must be followed by a variable."));
+                        program_error("&REST/&BODY must be followed by a variable.");
                       }
                   }
                 else if (obj == Symbol.AND_ENVIRONMENT)
                   {
                     if (type == LambdaListType.ORDINARY)
-                      error(new ProgramError("&ENVIRONMENT not allowed in ordinary lambda lists."));
+                      program_error("&ENVIRONMENT not allowed in ordinary lambda lists.");
                     remaining = remaining.cdr();
                     envVar = (Symbol) remaining.car();
                     envParam = new EnvironmentParam(envVar, isSpecial(envVar, specials));
@@ -304,8 +301,7 @@ public class ArgumentListProcessor {
                       {
                         if (state != STATE_REQUIRED)
                           {
-                            error(new ProgramError(
-                              "required parameters cannot appear after &REST/&BODY."));
+                            program_error("required parameters cannot appear after &REST/&BODY.");
                           }
                         if (required == null)
                           required = new ArrayList<Param>();
@@ -545,8 +541,8 @@ public class ArgumentListProcessor {
   }
   
   private static void invalidParameter(LispObject obj) {
-    error(new ProgramError(obj.princToString() +
-                         " may not be used as a variable in a lambda list."));
+    program_error(obj.princToString()
+                  + " may not be used as a variable in a lambda list.");
   }
 
   private Symbol[] extractVariables()
@@ -566,7 +562,7 @@ public class ArgumentListProcessor {
   /** Internal class implementing the argument list to lambda list matcher.
    * Because we have two implementations - a fast one and a slower one - we
    * need this abstract super class */
-  private static abstract class ArgumentMatcher {
+  private static abstract class ArgumentMatcher implements Serializable {
       abstract LispObject[] match(LispObject[] args, Environment _environment,
               Environment env, LispThread thread);
   }
@@ -618,8 +614,8 @@ public class ArgumentListProcessor {
                           if (k.keyword == key)
                               continue next_key;
 
-                      error(new ProgramError("Unrecognized keyword argument " +
-                                              key.printObject()));
+                      program_error("Unrecognized keyword argument "
+                                    + key.printObject() + ".");
                   }
               }
         } 
@@ -702,8 +698,8 @@ public class ArgumentListProcessor {
                 arglist.consume();
                 
                 if (key != Keyword.ALLOW_OTHER_KEYS)
-                    error(new ProgramError("Invalid keyword argument " + key.printObject()));
-                
+                    program_error("Invalid keyword argument "
+                                  + key.printObject() + ".");
                 allowOtherKeysValue = null;
             }
             
@@ -756,7 +752,7 @@ public class ArgumentListProcessor {
       /** Asserts the number of remaining arguments is even. */
       void assertRemainderKeywords() {
           if (((len - argsConsumed) & 1) == 1)
-              error(new ProgramError("Odd number of keyword arguments."));
+              program_error("Odd number of keyword arguments.");
       }
       
       /** Returns the next unconsumed value from the argument set, or 'null'
@@ -809,7 +805,7 @@ public class ArgumentListProcessor {
    * - KeywordParam
    * - AuxParam
    * */
-  public static abstract class Param {
+  public static abstract class Param implements Serializable {
       
       /** Assigns values to be bound to the correcsponding variables to the
        * array, using 'index' as the next free slot, consuming any required
