@@ -2,7 +2,7 @@
  * BuiltInClass.java
  *
  * Copyright (C) 2003-2007 Peter Graves
- * $Id: BuiltInClass.java 13769 2012-01-12 19:22:42Z rschlatte $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -192,8 +192,9 @@ public class BuiltInClass extends LispClass
   public static final LispClass CONCATENATED_STREAM =
     addClass(Symbol.CONCATENATED_STREAM,
              new StructureClass(Symbol.CONCATENATED_STREAM, list(SYSTEM_STREAM)));
-
-
+  public static final LispClass CLOS_STREAM =
+    addClass(Symbol.CLOS_STREAM,
+             new StructureClass(Symbol.CLOS_STREAM, list(SYSTEM_STREAM)));
 
     // Implementation defined streams
   public static final LispClass SOCKET_STREAM =
