@@ -150,6 +150,11 @@ public final class Cons extends LispObject implements java.io.Serializable
     return this;
   }
 
+  @Override
+  public int hashCode() {
+    return this.sxhash();
+  }
+  
   @Override
   public final int sxhash()
   {
