@@ -2,7 +2,7 @@
  * ControlError.java
  *
  * Copyright (C) 2003-2005 Peter Graves
- * $Id: ControlError.java 12288 2009-11-29 22:00:12Z vvoutilainen $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -46,7 +46,7 @@ public final class ControlError extends LispError
     public ControlError(String message)
     {
         super(StandardClass.CONTROL_ERROR);
-        setFormatControl(message);
+        setFormatControl(message.replaceAll("~","~~"));
         setFormatArguments(NIL);
     }
 
