@@ -33,9 +33,11 @@
 
 package org.armedbear.lisp;
 
+import java.io.Serializable;
+
 import static org.armedbear.lisp.Lisp.*;
 
-public final class Environment extends LispObject
+public final class Environment extends LispObject implements Serializable
 {
   Binding vars;
   FunctionBinding lastFunctionBinding;
@@ -124,6 +126,16 @@ public final class Environment extends LispObject
       return lookup(symbol, vars);
   }
 
+  public Binding getOuterMostBlock() {
+    Binding binding = blocks;
+    Binding result = binding;
+    while (binding != null) {
+      result = binding;
+      binding = binding.next;
+    }
+    return result;
+  }
+
   public Binding getBinding(LispObject symbol) {
     return getBinding(symbol, vars);
   }
@@ -358,8 +370,15 @@ public final class Environment extends LispObject
             LispObject result = NIL;
             for (Binding binding = env.vars;
                  binding != null; binding = binding.next)
-              if (binding.specialp)
-                result = result.push(binding.symbol);
+              if (binding.specialp) {
+                LispObject symbolValue = ((Symbol)binding.symbol).symbolValueNoThrow();
+                if (symbolValue != null) {
+                  result = result.push(new Cons(binding.symbol, symbolValue));
+                }
+                else {
+                  result = result.push(binding.symbol);
+                }
+              }
               else
                 result = result.push(new Cons(binding.symbol, binding.value));
             return result.nreverse();
