@@ -76,12 +76,11 @@ public final class Fixnum extends LispInteger
   @Override
   public Object javaInstance(Class c)
   {
-    String cn = c.getName();
-    if (cn.equals("java.lang.Byte") || cn.equals("byte"))
+    if (c == Byte.class || c == byte.class)
       return Byte.valueOf((byte)value);
-    if (cn.equals("java.lang.Short") || cn.equals("short"))
+    if (c == Short.class || c == short.class)
       return Short.valueOf((short)value);
-    if (cn.equals("java.lang.Long") || cn.equals("long"))
+    if (c == Long.class || c == long.class)
       return Long.valueOf((long)value);
     return javaInstance();
   }
@@ -501,8 +500,14 @@ public final class Fixnum extends LispInteger
       }
     catch (ArithmeticException e)
       {
-        if (obj.zerop())
-          return error(new DivisionByZero());
+        if (obj.zerop()) {
+          LispObject operands = new Cons(this, new Cons(obj));
+          LispObject args = new Cons(Keyword.OPERATION,
+                                     new Cons(Symbol.SLASH,
+                                              new Cons(Keyword.OPERANDS,
+                                                       new Cons(operands))));
+          return error(new DivisionByZero(args));
+        }
         return error(new ArithmeticError(e.getMessage()));
       }
   }
@@ -718,8 +723,14 @@ public final class Fixnum extends LispInteger
       }
     catch (ArithmeticException e)
       {
-        if (obj.zerop())
-          return error(new DivisionByZero());
+        if (obj.zerop()) {
+          LispObject operands = new Cons(this, new Cons(obj));
+          LispObject args = new Cons(Keyword.OPERATION,
+                                     new Cons(Symbol.TRUNCATE,
+                                              new Cons(Keyword.OPERANDS,
+                                                       new Cons(operands))));
+          return error(new DivisionByZero(args));
+        }
         else
           return error(new ArithmeticError(e.getMessage()));
       }
@@ -904,7 +915,7 @@ public final class Fixnum extends LispInteger
     if (y.compareTo(BigInteger.ZERO) == 0)
       // No need to test base here; CLHS says 0^0 == 1.
       return Fixnum.getInstance(1);
-      
+
     int x = this.value;
 
     if (x == 0)
