@@ -34,16 +34,22 @@
 package org.armedbear.lisp;
 
 import java.lang.ref.WeakReference;
+import java.lang.reflect.Method;
+import java.lang.reflect.InvocationTargetException;
 import static org.armedbear.lisp.Lisp.*;
 
 import java.util.Iterator;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.atomic.AtomicInteger;
-
+import java.util.Stack; 
 import java.text.MessageFormat;
 
-public final class LispThread extends LispObject
+import java.util.concurrent.ThreadFactory;
+
+public final class LispThread
+  extends LispObject
+  implements org.armedbear.lisp.protocol.Inspectable
 {
     // use a concurrent hashmap: we may want to add threads
     // while at the same time iterating the hash
@@ -79,12 +85,75 @@ public final class LispThread extends LispObject
     private Symbol wrapper =
         PACKAGE_THREADS.intern("THREAD-FUNCTION-WRAPPER");
 
+    /** Whether the wrapped thread is virtual */
+    boolean isVirtual;
+
+    public LispObject getParts() {
+      return NIL
+        .push(new Cons("Wrapped Java thread",
+                       JavaObject.getInstance(javaThread)));
+    }
+
+    /** Stack holding bindings for evaluated functions */
+    public Stack<Environment>envStack = new Stack<Environment>();
+
     LispThread(Thread javaThread)
     {
         this.javaThread = javaThread;
         name = new SimpleString(javaThread.getName());
     }
 
+  public static boolean virtualThreadingAvailable() {
+    try {
+      Method m 
+        = java.lang.Thread.class
+        .getDeclaredMethod("startVirtualThread", java.lang.Runnable.class);
+      return true;
+    } catch (NoSuchMethodException e2) {
+      // This is the case in non-Loom JVMs
+    } catch (SecurityException e3) {
+      Debug.trace("SecurityException caught introspecting threading interface: " + e3.toString());
+    }
+    return false;
+  }
+
+  static ThreadFactory virtualThreadFactory;
+  static Method newThread;
+
+  static {
+    try {
+      Object ofVirtual
+        = Thread.class.getMethod("ofVirtual").invoke(null);
+      Method factoryMethod
+        = Class.forName("java.lang.Thread$Builder").getMethod("factory");
+      virtualThreadFactory
+        = (ThreadFactory) factoryMethod.invoke(ofVirtual);
+      newThread  
+        = virtualThreadFactory.getClass()
+        .getMethod("newThread", java.lang.Runnable.class);
+      newThread.setAccessible(true);
+    } catch (Exception e) {
+      if (virtualThreadingAvailable()) {
+        Debug.trace("Failed to introspect virtual threading methods: " + e);
+      }
+    }
+  }
+
+  public static Symbol NATIVE_THREADS = internKeyword("NATIVE");
+  public static Symbol VIRTUAL_THREADS = internKeyword("VIRTUAL");
+
+  static {
+    Symbol._THREADING_MODEL_.initializeSpecial(NATIVE_THREADS);
+    final String summary
+      = "The current type of threads created via MAKE-THREAD";
+    final String doc
+      = "Meaningful values are either :NATIVE, the default, or the keyword :VIRTUAL."
+      + " The ability to invoke virtual threads at runtime is indicated by the presence of :VIRTUAL-THREADS"
+      + " in CL:*FEATURES*.";
+    Symbol._THREADING_MODEL_.setDocumentation(new SimpleString(summary + "\n" + doc),
+                                              Symbol.VARIABLE);
+  }
+
     LispThread(final Function fun, LispObject name)
     {
         Runnable r = new Runnable() {
@@ -116,13 +185,37 @@ public final class LispThread extends LispObject
                 }
             }
         };
-        javaThread = new Thread(r);
         this.name = name;
-        map.put(javaThread, this);
-        if (name != NIL)
-            javaThread.setName(name.getStringValue());
-        javaThread.setDaemon(true);
-        javaThread.start();
+
+        Thread thread = null;
+        
+        if (Symbol._THREADING_MODEL_.getSymbolValue().equals(NATIVE_THREADS)) {
+          thread = new Thread(r);
+          isVirtual = false;
+        } else {
+          try {
+            thread = (Thread)newThread.invoke(virtualThreadFactory, r);
+            isVirtual = true;
+          } catch (Exception e) {
+            thread = new Thread(r);
+            isVirtual =  false;
+            Debug.trace("Falling back to native thread creation after virtual thread failed:", e);
+          }
+        }
+        if (thread == null) {
+          Debug.trace("Failed to create java.lang.Thread");
+          javaThread = null;
+        } else {
+          if (name != NIL) {
+            thread.setName(name.getStringValue());
+          }
+          thread.setDaemon(true);
+
+          javaThread = thread;
+          map.put(javaThread, this);
+
+          javaThread.start();
+        }
     }
 
     public StackTraceElement[] getJavaStackTrace() {
@@ -454,7 +547,7 @@ public final class LispThread extends LispObject
                 thread.specials[index] = null;
             }
 
-            freeSpecialIndices.add(new Integer(index));
+            freeSpecialIndices.add(Integer.valueOf(index));
         }
     }
 
@@ -654,12 +747,11 @@ public final class LispThread extends LispObject
             this.next = next;
         }
         public LispObject getParts() {
-        Cons result = new Cons(NIL);
-        return result
-          .push(new Symbol("INITIAL-SEGMENT-SIZE"))
-            .push(LispInteger.getInstance(LispThread.INITIAL_SEGMENT_SIZE))
-          .push(new Symbol("SEGMENT-SIZE"))
-            .push(LispInteger.getInstance(LispThread.SEGMENT_SIZE)).nreverse();
+        return NIL
+          .push(new Cons(new Symbol("INITIAL-SEGMENT-SIZE"),
+                         LispInteger.getInstance(LispThread.INITIAL_SEGMENT_SIZE)))
+          .push(new Cons(new Symbol("SEGMENT-SIZE"),
+                         LispInteger.getInstance(LispThread.SEGMENT_SIZE)));
         }
     }
     
@@ -721,6 +813,7 @@ public final class LispThread extends LispObject
                 StackMarker marker = (StackMarker) stackObj;
                 int numArgs = marker.getNumArgs();
                 LispStackFrame frame = new LispStackFrame(stk, framePos - numArgs - STACK_FRAME_EXTRA, numArgs);
+                frame.thread = this;
                 stk[framePos - 1] = frame;
                 if (prev != null) {
                     prev.setNext(frame);
@@ -795,9 +888,11 @@ public final class LispThread extends LispObject
         stack[stackPtr + 1] = STACK_MARKER_0;
         stackPtr += STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute();
         }
         finally {
+            envStack.pop() ; 
             popStackFrame(0);
         }
     }
@@ -811,9 +906,11 @@ public final class LispThread extends LispObject
         stack[stackPtr + 2] = STACK_MARKER_1;
         stackPtr += 1 + STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute(arg);
         }
         finally {
+            envStack.pop() ; 
             popStackFrame(1);
         }
     }
@@ -829,9 +926,11 @@ public final class LispThread extends LispObject
         stack[stackPtr + 3] = STACK_MARKER_2;
         stackPtr += 2 + STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute(first, second);
         }
         finally {
+            envStack.pop() ; 
             popStackFrame(2);
         }
     }
@@ -848,9 +947,11 @@ public final class LispThread extends LispObject
         stack[stackPtr + 4] = STACK_MARKER_3;
         stackPtr += 3 + STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute(first, second, third);
         }
         finally {
+            envStack.pop() ; 
             popStackFrame(3);
         }
     }
@@ -869,9 +970,11 @@ public final class LispThread extends LispObject
         stack[stackPtr + 5] = STACK_MARKER_4;
         stackPtr += 4 + STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute(first, second, third, fourth);
         }
         finally {
+            envStack.pop() ; 
             popStackFrame(4);
         }
     }
@@ -891,9 +994,11 @@ public final class LispThread extends LispObject
         stack[stackPtr + 6] = STACK_MARKER_5;
         stackPtr += 5 + STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute(first, second, third, fourth, fifth);
         }
         finally {
+            envStack.pop() ; 
             popStackFrame(5);
         }
     }
@@ -915,9 +1020,11 @@ public final class LispThread extends LispObject
         stack[stackPtr + 7] = STACK_MARKER_6;
         stackPtr += 6 + STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute(first, second, third, fourth, fifth, sixth);
         }
         finally {
+            envStack.pop() ; 
             popStackFrame(6);
         }
     }
@@ -940,10 +1047,12 @@ public final class LispThread extends LispObject
         stack[stackPtr + 8] = STACK_MARKER_7;
         stackPtr += 7 + STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute(first, second, third, fourth, fifth, sixth,
                                     seventh);
         }
         finally {
+            envStack.pop();
             popStackFrame(7);
         }
     }
@@ -967,10 +1076,12 @@ public final class LispThread extends LispObject
         stack[stackPtr + 9] = STACK_MARKER_8;
         stackPtr += 8 + STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute(first, second, third, fourth, fifth, sixth,
                                     seventh, eighth);
         }
         finally {
+            envStack.pop() ; 
             popStackFrame(8);
         }
     }
@@ -983,9 +1094,11 @@ public final class LispThread extends LispObject
         stack[stackPtr + args.length + 1] = new StackMarker(args.length);
         stackPtr += args.length + STACK_FRAME_EXTRA;
         try {
+            envStack.push(new Environment(null,NIL,function));
             return function.execute(args);
         }
         finally {
+            envStack.pop() ; 
             popStackFrame(args.length);
         }
     }
@@ -1134,10 +1247,17 @@ public final class LispThread extends LispObject
             sb.append(name.getStringValue());
             sb.append("\"");
         }
+        if (isVirtual) {
+          sb.append(" virtual");
+        } else {
+          sb.append(" native");
+        }
         return unreadableString(sb.toString());
     }
 
-    @DocString(name="make-thread", args="function &key name")
+    @DocString(name="make-thread",
+               args="function &key name",
+               doc="Create a thread of execution running FUNCTION possibly named NAME")
     private static final Primitive MAKE_THREAD =
         new Primitive("make-thread", PACKAGE_THREADS, true, "function &key name")
     {
@@ -1163,8 +1283,9 @@ public final class LispThread extends LispObject
         }
     };
 
-    @DocString(name="threadp", args="object",
-    doc="Boolean predicate testing if OBJECT is a thread.")
+    @DocString(name="threadp",
+               args="object",
+               doc="Boolean predicate returning non-nil if OBJECT is a lisp thread")
     private static final Primitive THREADP =
         new Primitive("threadp", PACKAGE_THREADS, true)
     {
@@ -1175,11 +1296,12 @@ public final class LispThread extends LispObject
         }
     };
 
-    @DocString(name="thread-alive-p", args="thread",
-    doc="Returns T if THREAD is alive.")
+    @DocString(name="thread-alive-p",
+               args="thread",
+               doc="Returns T if THREAD is alive.")
     private static final Primitive THREAD_ALIVE_P =
-        new Primitive("thread-alive-p", PACKAGE_THREADS, true, "thread",
-              "Boolean predicate whether THREAD is alive.")
+      new Primitive("thread-alive-p", PACKAGE_THREADS, true, "thread",
+                    "Boolean predicate whether THREAD is alive.")
     {
         @Override
         public LispObject execute(LispObject arg)
@@ -1195,8 +1317,9 @@ public final class LispThread extends LispObject
         }
     };
 
-    @DocString(name="thread-name", args="thread",
-    doc="Return the name of THREAD, if it has one.")
+    @DocString(name="thread-name",
+               args="thread",
+               doc="Return the name of THREAD, if it has one.")
     private static final Primitive THREAD_NAME =
         new Primitive("thread-name", PACKAGE_THREADS, true)
     {
@@ -1211,13 +1334,15 @@ public final class LispThread extends LispObject
     };
 
     private static final Primitive THREAD_JOIN =
-        new Primitive("thread-join", PACKAGE_THREADS, true, "thread",
-                      "Waits for thread to finish.")
+      new Primitive("thread-join", PACKAGE_THREADS, true, "thread",
+                    "Waits for THREAD to die before resuming execution\n"
+                    + "Returns the result of the joined thread as its primary value.\n"
+                    + "Returns T if the joined thread finishes normally or NIL if it was interrupted.")
     {
         @Override
         public LispObject execute(LispObject arg)
         {
-            // join the thread, and returns it's value.  The second return
+            // join the thread, and returns its value.  The second return
             // value is T if the thread finishes normally, NIL if its 
             // interrupted. 
             if (arg instanceof LispThread) {                
@@ -1361,6 +1486,7 @@ public final class LispThread extends LispObject
             for (int i = args.length; i-- > 2;)
                 funArgs = new Cons(args[i], funArgs);
             thread.interrupt(fun, funArgs);
+            setInterrupted(thread,true);
             return T;
         }
     };
