@@ -2,7 +2,7 @@
  * Package.java
  *
  * Copyright (C) 2002-2007 Peter Graves <peter@armedbear.org>
- * $Id: Package.java 14570 2013-07-22 13:21:06Z mevenson $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -460,7 +460,7 @@ public final class Package extends LispObject implements java.io.Serializable
                             sb.append(sym.getQualifiedName());
                             sb.append(" and ");
                             sb.append(s.getQualifiedName());
-                            return error(new PackageError(sb.toString()));
+                            return error(new PackageError(sb.toString(), this));
                         }
                     }
                     usedPackages = usedPackages.cdr();
@@ -500,7 +500,7 @@ public final class Package extends LispObject implements java.io.Serializable
             sb.append(" is already accessible in package ");
             sb.append(name);
             sb.append('.');
-            error(new PackageError(sb.toString()));
+            error(new PackageError(sb.toString(), this));
         }
         internalSymbols.put(symbol.name.toString(), symbol);
         if (symbol.getPackage() == NIL)
@@ -519,7 +519,7 @@ public final class Package extends LispObject implements java.io.Serializable
                 sb.append(" is not accessible in package ");
                 sb.append(name);
                 sb.append('.');
-                error(new PackageError(sb.toString()));
+                error(new PackageError(sb.toString(), this));
                 return;
             }
             internalSymbols.put(symbol.name.toString(), symbol);
@@ -540,7 +540,7 @@ public final class Package extends LispObject implements java.io.Serializable
                             sb.append(" is already accessible in package ");
                             sb.append(pkg.getName());
                             sb.append('.');
-                            error(new PackageError(sb.toString()));
+                            error(new PackageError(sb.toString(), pkg));
                             return;
                         }
                     }
@@ -559,7 +559,7 @@ public final class Package extends LispObject implements java.io.Serializable
         sb.append(" is not accessible in package ");
         sb.append(name);
         sb.append('.');
-        error(new PackageError(sb.toString()));
+        error(new PackageError(sb.toString(), this));
     }
 
     public synchronized void unexport(final Symbol symbol)
@@ -573,7 +573,7 @@ public final class Package extends LispObject implements java.io.Serializable
         sb.append(symbol.getQualifiedName());
         sb.append(" is not accessible in package ");
         sb.append(name);
-        error(new PackageError(sb.toString()));
+        error(new PackageError(sb.toString(), this));
       }
     }
 
@@ -646,7 +646,7 @@ public final class Package extends LispObject implements java.io.Serializable
                     {
                         error(new PackageError("A symbol named " + symbol.getName() +
                                                 " is already accessible in package " +
-                                                name + "."));
+                                                name + ".", this));
                         return;
                     }
                 }
@@ -928,11 +928,11 @@ public final class Package extends LispObject implements java.io.Serializable
     }
 
     public Object readResolve() throws java.io.ObjectStreamException {
-	Package pkg = findPackage(name);
-	if(pkg != null) {
-	    return pkg;
-	} else {
-	    return error(new PackageError(name + " is not the name of a package."));
-	}
+        Package pkg = findPackage(name);
+        if(pkg != null) {
+            return pkg;
+        } else {
+            return error(new PackageError(name + " is not the name of a package.", new SimpleString(name)));
+        }
     }
 }
