@@ -2,7 +2,7 @@
  * PackageFunctions.java
  *
  * Copyright (C) 2003-2005 Peter Graves
- * $Id: PackageFunctions.java 14431 2013-03-10 15:52:36Z rschlatte $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -363,10 +363,11 @@ public final class PackageFunctions
             if (nicknames != NIL) {
                 LispObject list = nicknames;
                 while (list != NIL) {
-                    String nick = javaString(list.car());
+                    LispObject lispNick = list.car();
+                    String nick = javaString(lispNick);
                     if (currentpkg.findPackage(nick) != null) {
                         return error(new PackageError("A package named " + nick +
-                                                       " already exists."));
+                                                       " already exists.", lispNick));
                     }
                     list = list.cdr();
                 }
