@@ -2,7 +2,7 @@
  * ParseError.java
  *
  * Copyright (C) 2003-2005 Peter Graves
- * $Id: ParseError.java 12288 2009-11-29 22:00:12Z vvoutilainen $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,7 +40,7 @@ public final class ParseError extends LispError
     public ParseError(String message)
     {
         super(StandardClass.PARSE_ERROR);
-        setFormatControl(message);
+        setFormatControl(message.replaceAll("~","~~"));
         setFormatArguments(NIL);
     }
 
