@@ -31,15 +31,12 @@
 
 (in-package #:system)
 
-;; Adapted from SBCL.
-(defun cfp-output-file-default (input-file)
-  (let* ((defaults (merge-pathnames input-file *default-pathname-defaults*))
-	 (retyped (make-pathname :type *compile-file-type* :defaults defaults)))
-    retyped))
-
-(defun compile-file-pathname (input-file
-                              &key
-                              (output-file (cfp-output-file-default
-                                            input-file))
-                              &allow-other-keys)
-  (merge-pathnames output-file (merge-pathnames input-file) nil))
+(defun compile-file-pathname (input-file &key output-file &allow-other-keys)
+  (let ((defaults (make-pathname :type *compile-file-type*
+                                 :defaults (merge-pathnames input-file))))
+    (cond ((null output-file) defaults)
+          (t (let ((computed-file-pathname (merge-pathnames output-file defaults)))
+               (if (equal (pathname-directory computed-file-pathname)
+                          (pathname-directory (make-pathname :defaults input-file)))
+                   computed-file-pathname
+                   (merge-pathnames output-file *default-pathname-defaults*)))))))
