@@ -1,7 +1,7 @@
 ;;; compiler-error.lisp
 ;;;
 ;;; Copyright (C) 2003-2005 Peter Graves
-;;; $Id: compiler-error.lisp 14018 2012-07-29 11:32:09Z ehuelsmann $
+;;; $Id$
 ;;;
 ;;; This program is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU General Public License
@@ -35,14 +35,16 @@
           compiler-style-warn
           compiler-warn
           compiler-error
+          compiler-bytecode-length-error
           internal-compiler-error
           compiler-unsupported))
 
 (defvar *compiler-error-context* nil)
 
-(define-condition compiler-error (error))
-(define-condition internal-compiler-error (compiler-error))
-(define-condition compiler-unsupported-feature-error (compiler-error))
+(define-condition compiler-error (error) ())
+(define-condition compiler-bytecode-length-error (error) ())
+(define-condition internal-compiler-error (compiler-error) ())
+(define-condition compiler-unsupported-feature-error (compiler-error) ())
 
 (defun compiler-style-warn (format-control &rest format-arguments)
   (warn 'style-warning
@@ -60,7 +62,7 @@
          :format-arguments format-arguments))
 
 (defun internal-compiler-error (format-control &rest format-arguments)
-  (cerror "Eventually use interpreted form instead" 
+  (cerror "Eventually use interpreted form instead"
           'internal-compiler-error
           :format-control format-control
           :format-arguments format-arguments))
