@@ -1,7 +1,7 @@
 ;;; defstruct.lisp
 ;;;
 ;;; Copyright (C) 2003-2007 Peter Graves <peter@armedbear.org>
-;;; $Id: defstruct.lisp 14162 2012-10-03 21:28:11Z ehuelsmann $
+;;; $Id$
 ;;;
 ;;; This program is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU General Public License
@@ -353,19 +353,23 @@
         (type (dsd-type slot)))
     (cond ((eq *dd-type* 'list)
            `((declaim (ftype (function * ,type) ,accessor-name))
+             (record-source-information-for-type ',accessor-name '(:structure-reader ,*dd-name*))
              (setf (symbol-function ',accessor-name)
                    (make-list-reader ,index))))
           ((or (eq *dd-type* 'vector)
                (and (consp *dd-type*) (eq (car *dd-type*) 'vector)))
            `((declaim (ftype (function * ,type) ,accessor-name))
+             (record-source-information-for-type ',accessor-name '(:structure-reader ,*dd-name*))
              (setf (symbol-function ',accessor-name)
                    (make-vector-reader ,index))
+             (record-source-information-for-type ',accessor-name '(:structure-reader ,*dd-name*))
              (define-source-transform ,accessor-name (instance)
                `(aref (truly-the ,',*dd-type* ,instance) ,,index))))
           (t
            `((declaim (ftype (function * ,type) ,accessor-name))
              (setf (symbol-function ',accessor-name)
                    (make-structure-reader ,index ',*dd-name*))
+             (record-source-information-for-type ',accessor-name '(:structure-reader ,*dd-name*))
              (define-source-transform ,accessor-name (instance)
                ,(if (eq type 't)
                     ``(structure-ref (the ,',*dd-name* ,instance) ,,index)
@@ -395,17 +399,20 @@
   (let ((accessor-name (dsd-reader slot))
         (index (dsd-index slot)))
     (cond ((eq *dd-type* 'list)
-           `((setf (get ',accessor-name 'setf-function)
+           `((record-source-information-for-type '(setf ,accessor-name) '(:structure-writer ,*dd-name*))
+             (setf (get ',accessor-name 'setf-function)
                    (make-list-writer ,index))))
           ((or (eq *dd-type* 'vector)
                (and (consp *dd-type*) (eq (car *dd-type*) 'vector)))
            `((setf (get ',accessor-name 'setf-function)
                    (make-vector-writer ,index))
+             (record-source-information-for-type '(setf ,accessor-name) '(:structure-writer ,*dd-name*))
              (define-source-transform (setf ,accessor-name) (value instance)
                `(aset (truly-the ,',*dd-type* ,instance) ,,index ,value))))
           (t
            `((setf (get ',accessor-name 'setf-function)
                    (make-structure-writer ,index ',*dd-name*))
+             (record-source-information-for-type '(setf ,accessor-name) '(:structure-writer ,*dd-name*))
              (define-source-transform (setf ,accessor-name) (value instance)
                `(structure-set (the ,',*dd-name* ,instance)
                                ,,index ,value)))))))
@@ -424,12 +431,15 @@
 (defun define-copier ()
   (when *dd-copier*
     (cond ((eq *dd-type* 'list)
-           `((setf (fdefinition ',*dd-copier*) #'copy-list)))
+           `((declaim (ftype (function (list) list) ,*dd-copier*))
+             (setf (fdefinition ',*dd-copier*) #'copy-list)))
           ((or (eq *dd-type* 'vector)
                (and (consp *dd-type*) (eq (car *dd-type*) 'vector)))
-           `((setf (fdefinition ',*dd-copier*) #'copy-seq)))
+           `((declaim (ftype (function (vector) vector) ,*dd-copier*))
+             (setf (fdefinition ',*dd-copier*) #'copy-seq)))
           (t
-           `((setf (fdefinition ',*dd-copier*) #'copy-structure))))))
+           `((declaim (ftype (function (T) T) ,*dd-copier*))
+             (setf (fdefinition ',*dd-copier*) #'copy-structure))))))
 
 (defun define-print-function ()
   (cond (*dd-print-function*
@@ -695,6 +705,7 @@
                              :slots ',*dd-slots*
                              :inherited-accessors ',*dd-inherited-accessors*
                              :documentation ',*dd-documentation*))
+       (record-source-information-for-type ',*dd-name* :structure)
        ,@(define-constructors)
        ,@(define-predicate)
        ,@(define-access-functions)
