@@ -29,7 +29,7 @@
 ;;; obligated to do so.  If you do not wish to do so, delete this
 ;;; exception statement from your version.
 
-(in-package "JVM")
+(in-package :jvm)
 (require '#:compiler-types)
 
 #|
@@ -105,7 +105,8 @@ representation to use.
 |#
 
 (defstruct (jvm-class-name (:conc-name class-)
-                           (:constructor %make-jvm-class-name))
+                           (:constructor %make-jvm-class-name)
+                           (:print-object %print-jvm-class-name))
   "Used for class identification.
 
 The caller should instantiate only one `class-name' per class, as they are
@@ -120,6 +121,10 @@ This class is used to abstract from the difference."
   ;; name comparisons to be EQ: all classes should exist only once,
   )
 
+(defun %print-jvm-class-name (name stream)
+  (print-unreadable-object (name stream :type t)
+    (write-string (class-name-internal name) stream)))
+
 (defun make-jvm-class-name (name)
   "Creates a `class-name' structure for the class or interface `name'.
 
@@ -127,7 +132,7 @@ This class is used to abstract from the difference."
 to 'internal' (JVM) representation by this function."
   (setf name (substitute #\/ #\. name))
   (%make-jvm-class-name :name-internal name
-			:ref (concatenate 'string "L" name ";")))
+                        :ref (concatenate 'string "L" name ";")))
 
 (defun class-array (class-name)
   "Returns a class-name representing an array of `class-name'.
@@ -145,7 +150,7 @@ and used on successive calls."
     (let ((name-and-ref (concatenate 'string "[" (class-ref class-name))))
       (setf (class-array-class class-name)
             (%make-jvm-class-name :name-internal name-and-ref
-				  :ref name-and-ref))))
+                                  :ref name-and-ref))))
   (class-array-class class-name))
 
 (defmacro define-class-name (symbol java-dotted-name &optional documentation)
@@ -238,13 +243,19 @@ type, or a `class-name' structure identifying a class or interface."
       (map-primitive-type field-type)
       (class-name-internal field-type)))
 
+(defun map-array-type (field-type)
+  (assert (eql :array (first field-type)))
+  (concatenate 'string "["
+   (internal-field-ref (second field-type))))
+
 (defun internal-field-ref (field-type)
   "Returns a string containing the JVM-internal representation of a reference
 to `field-type', which should either be a symbol identifying a primitive
 type, or a `class-name' structure identifying a class or interface."
-  (if (symbolp field-type)
-      (map-primitive-type field-type)
-      (class-ref field-type)))
+  (etypecase field-type
+    (symbol (map-primitive-type field-type))
+    (cons (map-array-type field-type))
+    (jvm-class-name (class-ref field-type))))
 
 (defun descriptor (return-type &rest argument-types)
   "Returns a string describing the `return-type' and `argument-types'
@@ -262,7 +273,7 @@ in JVM-internal representation."
       (princ #\) s)
       (princ ret-string s))
     str)
-;;  (format nil "(~{~A~})~A" 
+;;  (format nil "(~{~A~})~A"
 ;;          (internal-field-ref return-type))
   )
 
@@ -798,7 +809,7 @@ modfies the interfaces members with a list of the references to the
 corresponding pool indices."
   (let ((interface-refs nil))
     (dolist (interface (class-file-interfaces class))
-      (push 
+      (push
        (pool-add-class (class-file-constants class)
                        interface)
        interface-refs))
@@ -1099,7 +1110,7 @@ Returns NIL if the attribute isn't found."
 
 
 (defstruct (jvm-method (:constructor %make-jvm-method)
-		       (:conc-name method-))
+                       (:conc-name method-))
   "Holds information on the properties of methods in the class(-file)."
   access-flags
   name
@@ -1124,8 +1135,8 @@ be one of two keyword identifiers to identify special methods:
 (defun make-jvm-method (name return args &key (flags '(:public)))
   "Creates a method for addition to a class file."
   (%make-jvm-method :descriptor (cons return args)
-		    :access-flags flags
-		    :name (map-method-name name)))
+                    :access-flags flags
+                    :name (map-method-name name)))
 
 (defun method-add-attribute (method attribute)
   "Add `attribute' to the list of attributes of `method',
@@ -1141,7 +1152,7 @@ returning the created attribute."
    (make-code-attribute (+ (length (cdr (method-descriptor method)))
                            (if (member :static (method-access-flags method))
                                0 1)) ;; 1 == implicit 'this'
-			optimize)))
+                        optimize)))
 
 (defun method-ensure-code (method &optional (optimize t))
   "Ensures the existence of a 'Code' attribute for the method,
@@ -1270,7 +1281,8 @@ has been finalized."
     (multiple-value-bind
           (c labels)
         (code-bytes c)
-      (assert (< 0 (length c) 65536))
+      (unless (< 0 (length c) 65536)
+        (signal 'sys:compiler-bytecode-length-error))
       (setf (code-code code) c
             (code-labels code) labels)))
 
@@ -1681,9 +1693,17 @@ appropriate reflective APIs.")
   (dolist (ann (annotations-list annotations))
     (finalize-annotation ann class)))
 
+;; See https://github.com/openjdk/jdk/blob/8758b554a089af98fdc3be4201d61278d5f01af3/src/java.base/share/classes/sun/reflect/generics/parser/SignatureParser.java#L295-L312
+(defun class-name->type-signature (class-name)
+  (concatenate 'string
+               '(#\L)
+               (substitute #\/ #\. class-name)
+               '(#\;)))
+
 (defun finalize-annotation (ann class)
   (setf (annotation-type ann)
-        (pool-add-class (class-file-constants class) (annotation-type ann)))
+        (pool-add-utf8 (class-file-constants class)
+                       (class-name->type-signature (annotation-type ann))))
   (dolist (elem (annotation-elements ann))
     (finalize-annotation-element elem class)))
 
@@ -1729,7 +1749,7 @@ be able to
 - find the method's code attribute
 - add code to the code attribute
 - finalize the code attribute contents (blocking it for further addition)
-- 
+-
 
 
 |#
