@@ -29,7 +29,7 @@
 ;;; obligated to do so.  If you do not wish to do so, delete this
 ;;; exception statement from your version.
 
-(in-package "JVM")
+(in-package :jvm)
 
 (export '(compile-defun *catch-errors* derive-compiler-type))
 
